package jpatests.helper;

import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import java.io.PrintStream;
import java.net.URL;
import java.util.Properties;
import java.util.Map;
import java.util.HashMap;

public class JpaTestBase {
    public static Map propertiesMap = null;
    public static String PROPERTIES_FILE = "META-INF/persistence.properties"; 

    protected EntityManagerFactory emf;
    protected EntityManager em;

    protected static PrintStream out = System.out;

    public synchronized static Map getPersistenceProperties(){
        if (propertiesMap == null){
            Properties properties = new Properties();
            URL url = Thread.currentThread().getContextClassLoader().getResource(PROPERTIES_FILE);
            propertiesMap = new HashMap();
            if (url != null){
                try{
                    properties.load(url.openStream());
                } catch (java.io.IOException exception){
                   throw new  RuntimeException("Error loading persistence.properties.", exception);
                }
                propertiesMap.putAll(properties);
            }
        }
        return propertiesMap;
    }

    public void createEntityManagerFactory(String unitName) {
        this.emf = Persistence.createEntityManagerFactory(unitName, getPersistenceProperties());
    }

    public EntityManager getEntityManager(){
        return em;
    }

    public void begin(){
        if(em == null)
            em = emf.createEntityManager();
        em.getTransaction().begin();
    }

    public void commit(){
        em.getTransaction().commit();
    }

    public void commitAndClear(){
        em.getTransaction().commit();
        em.clear();
    }

    public void commitAndClose(){
        em.getTransaction().commit();
        em.close();
        em = null;
    }

    public void rollback(){
        if(em != null){
            EntityTransaction tx = em.getTransaction();
            if(tx != null && tx.isActive())
                tx.rollback();
        }
    }

    public void closeAll(){
        if(em != null){
            EntityTransaction tx = em.getTransaction();
            if(tx != null && tx.isActive())
                tx.rollback();
            em.close();
        }
        if(emf != null)
            emf.close();
    }
    
    public static void println(String s){
        JpaTestBase.out.println("### " + s);
    }
}
