package jpatests.tests;

import org.testng.annotations.Test;
import jpatests.helper.JpaTestBase;
import jpatests.model.cascade.*;

import java.util.List;
import java.util.ArrayList;

/**
 * @author Wonseok Kim
 */
@Test
public class CascadeTest extends JpaTestBase {
    public void testCascadeMerge() throws Exception {
        createEntityManagerFactory("default");
        
        try {
            // normal case
            begin();
            EntityA a1 = new EntityA("a1");
            EntityB b1 = new EntityB("b1");
            EntityC c1 = new EntityC("c1");
            a1.setB(b1);
            List list = new ArrayList();
            list.add(c1);
            a1.setC(list);
            c1.setA(a1);
            
            EntityA ma1 = em.merge(a1); // cascade merge (persist if new)
            assert em.contains(ma1);
            assert em.contains(ma1.getB());
            for(EntityC c : ma1.getC()){
                assert em.contains(c);
            }
            commitAndClose();
            
            // if a is already managed, test merge propagation
            begin();
            ma1 = em.find(EntityA.class, "a1");

            EntityB b2 = new EntityB("b2");
            EntityC c2 = new EntityC("c2");
            ma1.setB(b2);
            List list2 = new ArrayList();
            list2.add(c2);
            ma1.setC(list2);
            c2.setA(ma1);
            
            ma1 = em.merge(ma1); // cascade merge
            assert em.contains(ma1);
            assert em.contains(ma1.getB());
            for(EntityC c : ma1.getC()){
                assert em.contains(c);
            }
            commitAndClose();

        } finally {
            closeAll();
        }
    }
}
